@@ -55,9 +55,7 @@ OMexample <- function(dir) {
 #' 
 OMinit <- function(name=NULL, ..., files=c('xlsx', 'rmd'), dir=NULL, overwrite=FALSE) {
   files <- match.arg(files, several.ok = TRUE)
-  
   if(is.null(dir)) dir <- getwd()
-  
   if (is.null(name)) stop("Require OM name", call.=FALSE)
   
   if (tolower(name) == 'example') {
@@ -89,8 +87,10 @@ OMinit <- function(name=NULL, ..., files=c('xlsx', 'rmd'), dir=NULL, overwrite=F
      inclasses <- unlist(lapply(InTemplates, class))
     if (!is.null(inclasses)) {
       # check if zip application exists
-      chck <- Sys.which("zip") # requires 'zip.exe' on file path
-      if (nchar(chck) <1) {
+      # chck <- Sys.which("zip") # requires 'zip.exe' on file path
+      chck <- Sys.getenv("R_ZIPCMD", "zip")
+      
+      if (!'zip' %in% chck) {
         message('zip application is required for templates. If a zip application is installed on your machine you may need to add it to the path. Try:')
         message('path <- Sys.getenv("PATH")')
         message('Sys.setenv("PATH" = paste(path, "path_to_zip.exe", sep = ";"))')
@@ -128,9 +128,9 @@ OMinit <- function(name=NULL, ..., files=c('xlsx', 'rmd'), dir=NULL, overwrite=F
       }
     }
   }
-
+  
   if ('xlsx' %in% files) {
-   
+    
     # Copy xlsx file over to working directory 
     # Copy the Excel File ####
     message("Creating ", name, " in ", dir)
@@ -152,34 +152,31 @@ OMinit <- function(name=NULL, ..., files=c('xlsx', 'rmd'), dir=NULL, overwrite=F
         if (!is.null(ObTemplates[objname])) {
           obj <- ObTemplates[objname][[1]]
           slots <- slotNames(obj)
-          
-          for (sl in seq_along(slots)) {
-            val <- slot(obj, slotNames(objname)[sl])
+          # ignore grad slots 
+          slots <- slots[!grepl("grad", slots)]
+          shtdata <- openxlsx::read.xlsx(wb, objname)
+          for (sl in seq_along(slots)) { 
+            row <- match(slots[sl], shtdata[,1])
+            val <- slot(obj, slots[sl])
             ln <- length(val)
-            if (ln >0 && !is.na(ln)) {
+            if (ln >0 && !is.na(val)) {
               df <- data.frame(t(val))
               openxlsx::writeData(wb, sheet = objname, x = df, 
-                                  startCol = 2, startRow = sl+1,
+                                  startCol = 2, startRow = row+1,
                                   colNames = FALSE, rowNames = FALSE, 
                                   withFilter = FALSE,
                                   keepNA = FALSE)         
-            }
-            
+            } 
           }
-          openxlsx::setColWidths(wb, sheet = objname, cols = 1, widths = 'auto')
+          # openxlsx::setColWidths(wb, sheet = objname, cols = 1, widths = 'auto')
         }
       }
       
       # OM tab not currently updated
       openxlsx::saveWorkbook(wb, file.path(dir,name), overwrite = TRUE)
-    
-      
     }
-    
-      
-    }
-    
-
+  }
+  
   if ('rmd' %in% files) { 
     # RMD File ####
     rmdname <- paste0(nameNoExt, '.rmd')
@@ -274,7 +271,7 @@ XL2OM <- function(name=NULL, cpars=NULL, msg=TRUE) {
   count <- 1
   tempObj <- vector("list", 4)
   for (obj in c("Stock", "Fleet", "Obs", "Imp")) {
-    sht <- as.data.frame(readxl::read_excel(name, sheet = obj, col_names = FALSE))
+    sht <- suppressMessages(as.data.frame(readxl::read_excel(name, sheet = obj, col_names = FALSE)))
     rows <- sht[,1] 
     rows <- rows[!rows == "Slot"]
     ind <- which(!rows %in% slotNames(obj))
@@ -301,7 +298,7 @@ XL2OM <- function(name=NULL, cpars=NULL, msg=TRUE) {
             Obs = tempObj[[3]], Imp=tempObj[[4]])
   
   # Read in the OM sheet
-  sht <- as.data.frame(readxl::read_excel(name, sheet = "OM", col_names = FALSE))
+  sht <- suppressMessages(as.data.frame(readxl::read_excel(name, sheet = "OM", col_names = FALSE)))
   dat <- sht # sht[,1:2] 
   dat <- dat[which(dat[,1] != "Slot"),]
   # if (ncol(sht)>2) warning("More than two columns found in Sheet OM. Values in columns C+ are ignored")
@@ -335,7 +332,7 @@ XL2OM <- function(name=NULL, cpars=NULL, msg=TRUE) {
       stop("'cpars' must be a list", call.=FALSE)
     }
   }
-  ChkObj(OM, FALSE)
+  tt <- ChkObj(OM, FALSE)
   if (msg) {
     message('OM successfully imported\n')
     message("Document OM slots in .rmd file (probably ", tools::file_path_sans_ext(name), ".rmd),
@@ -381,7 +378,8 @@ XL2OM <- function(name=NULL, cpars=NULL, msg=TRUE) {
 #' OMdoc(myOM)
 #' }
 OMdoc <- function(OM=NULL, rmd.source=NULL, overwrite=FALSE, out.file=NULL,  
-                  inc.plot=TRUE, render=TRUE, output="html_document", openFile=TRUE, quiet=FALSE,
+                  inc.plot=TRUE, render=TRUE, output="html_document", 
+                  openFile=TRUE, quiet=FALSE,
                   dir=NULL, ...) {
   # markdown compile options
   toc=TRUE; color="blue";  theme="flatly"
@@ -545,7 +543,6 @@ OMdoc <- function(OM=NULL, rmd.source=NULL, overwrite=FALSE, out.file=NULL,
   Pars <- NULL
   out <- NULL
   if (inc.plot) {
-    
     # --- Generate Historical Samples ----
     # Only required if the OM has changed #
     runSims <- FALSE
@@ -585,7 +582,7 @@ OMdoc <- function(OM=NULL, rmd.source=NULL, overwrite=FALSE, out.file=NULL,
           if (sum(changed)>0) runSims <- TRUE 
           if (sum(changed) == 0) {
             out <-  readRDS(file.path(dir,paste0('build/', fileName, 'Hist.dat')))
-            Pars <- c(out$SampPars, out$TSdata, out$MSYs)  
+            Pars <- c(out@AtAge, out@TSdata, out@Ref, out@SampPars)  
           }
         } else {
           file.remove(file.path(dir,paste0('build/',fileName, '.dat')))
@@ -610,7 +607,7 @@ OMdoc <- function(OM=NULL, rmd.source=NULL, overwrite=FALSE, out.file=NULL,
         OM2@nsim <- 48
       }
       out<- runMSE(OM2,Hist=T, parallel = FALSE, silent=TRUE, ...)
-      Pars <- c(out$SampPars, out$TSdata, out$MSYs)  
+      Pars <- c(out@AtAge, out@TSdata, out@Ref, out@SampPars)  
       saveRDS(out, file=file.path(dir,paste0('build/', fileName, 'Hist.dat')))
     }
   }
@@ -715,7 +712,6 @@ OMdoc <- function(OM=NULL, rmd.source=NULL, overwrite=FALSE, out.file=NULL,
 
   }
   
-  
   ## References ####
   message("writing Reference section")
   writeSection(class="References", OM, Pars, textIn, RMDfile, color=color, inc.plot=inc.plot)
@@ -759,10 +755,10 @@ OMdoc <- function(OM=NULL, rmd.source=NULL, overwrite=FALSE, out.file=NULL,
 Template <- function(type=c("Stock", "Fleet", "Obs", "Imp")) {
   type <- match.arg(type)
   if (type == "Stock") mat <- 
-      matrix(c("Mortality and age:  maxage, R0, M, M2, Mexp, Msd, Mgrad",
+      matrix(c("Mortality and age:  maxage, R0, M, M2, Mexp, Msd",
                "Recruitment: h, SRrel, Perr, AC",
                "Non-stationarity in stock productivity: Period, Amplitude",
-               "Growth: Linf, K, t0, LenCV, Ksd, Kgrad, Linfsd, Linfgrad",
+               "Growth: Linf, K, t0, LenCV, Ksd, Linfsd",
                "Maturity: L50, L50_95",
                "Stock depletion: D",
                "Length-weight conversion parameters: a, b",
@@ -797,6 +793,8 @@ Template <- function(type=c("Stock", "Fleet", "Obs", "Imp")) {
  
   # Check slots 
   Slots <- names(methods::getSlots(type))
+  # ignore grad slots 
+  Slots <- Slots[!grepl("grad", Slots)]
   for (X in Slots) {
     tt <- grep(paste0("\\<", X, "\\>"), mat) 
     if (X != "Name" && X != "Source" && X!="Species" && X!="Common_Name" && X!="Latitude" && X!='Longitude') {
@@ -1022,10 +1020,6 @@ writeCSV2 <- function(inobj, tmpfile = NULL, objtype = c("Stock", "Fleet",
   }
 }
 
-
-
-
-
 plotText <- function(OM, slots, RMDfile) {
   if (any(c("M", "h", "Linf", "L50", "D", "EffUpper", "qcv", "Vmaxlen", "DR") %in% slots)) {
     # slotstext <- paste("c(", paste(slots, sep=",", collapse = ","), ")")
@@ -1076,8 +1070,7 @@ plotSlot <- function(OM, Pars, slot) {
 
 plotSelHists <- function(OM, Pars, nsamp=3, col="darkgray", 
                          breaks=10, lwd=2) {
-  
-  
+
   ncol <- 3
   m <- layout(matrix(c(1,2,3,
                        4,5,6,
@@ -1182,7 +1175,7 @@ plotDep <- function(OM, Pars=NULL, nsim=48, nyears=50, proyears=50, nsamp=3, col
   if (is.null(Pars)) {
     OM <- updateMSE(OM) # update and add missing slots with default values
     out<- runMSE(OM,Hist=T)
-    Pars <- c(out$SampPars, out$TSdata, out$MSYs)
+    Pars <- c(out@SampPars, out@TSdata, out@Ref)
   }
   
   its <- sample(1:nsim, nsamp)
@@ -1193,8 +1186,9 @@ plotDep <- function(OM, Pars=NULL, nsim=48, nyears=50, proyears=50, nsamp=3, col
   op <- par(mar = c(3, 2, 2, 1), oma=c(2,3,2,1), las=1, no.readonly = TRUE)
   on.exit(par(op))
   
-  ssb0 <- matrix(rep(Pars$SSB0, nyears), nrow=nyears, byrow=TRUE)
-  dep <- Pars$SSB/ssb0
+  ssb0 <- matrix(rep(Pars$SSB0, nyears), nrow=nsim, ncol=nyears, byrow=FALSE)
+  dep <- t(Pars$SSB/ssb0)
+  
   ylim <- c(0, max(dep))
   matplot(dep,  type="l", bty="l", ylab="SB/SB0", xlab="Historical Years", xpd=NA, ylim=ylim)
   matplot(dep[, its],  type="l", bty="l", ylab="", xlab="", add=TRUE, lwd=4, col=1:nsamp, 
@@ -1249,11 +1243,10 @@ plotMat <- function(OM, Pars=NULL, nsim=48, nyears=50, proyears=50, nsamp=3, col
   matplot(Ls, Mat_len, type="l", bty="l", main="Maturity-at-length", lwd=lwd, lty=1, 
           ylab="Probability", xlab="Length", ylim=c(0,1), xpd=NA)
   
-  matplot(t(Pars$Mat_age[its,,nyears]), type="l", bty="l", main="Maturity-at-age", lwd=lwd, 
-          lty=1, axes=FALSE, xlim=c(0, Pars$maxage), ylab="", xlab="Age", ylim=c(0,1), xpd=NA)
+  matplot(t(Pars$Maturity[its,,nyears]), type="l", bty="l", main="Maturity-at-age", lwd=lwd, 
+          lty=1, axes=FALSE, xlim=c(0, max(Pars$maxage)), ylab="", xlab="Age", ylim=c(0,1), xpd=NA)
   axis(side=1)
   axis(side=2, labels=FALSE)
-  
 }
 
 
@@ -1311,9 +1304,9 @@ plotGrowth <- function(OM, Pars=NULL, nsim=48, nyears=50, proyears=50, nsamp=3,
   axis(side=1)  
   
   # Linfgrad 
-  hist2(Pars$Linfgrad, col=col, axes=FALSE, main="Linfgrad", breaks=breaks)
-  abline(v=Pars$Linfgrad[its], col=1:nsamp, lwd=lwd)
-  axis(side=1) 
+  # hist2(Pars$Linfgrad, col=col, axes=FALSE, main="Linfgrad", breaks=breaks)
+  # abline(v=Pars$Linfgrad[its], col=1:nsamp, lwd=lwd)
+  # axis(side=1) 
   
   # Ksd 
   hist2(Pars$Ksd, col=col, axes=FALSE, main="Ksd", breaks=breaks)
@@ -1321,9 +1314,9 @@ plotGrowth <- function(OM, Pars=NULL, nsim=48, nyears=50, proyears=50, nsamp=3,
   axis(side=1)  
   
   # Kgrad 
-  hist2(Pars$Kgrad, col=col, axes=FALSE, main="Kgrad", breaks=breaks)
-  abline(v=Pars$Kgrad[its], col=1:nsamp, lwd=lwd)
-  axis(side=1)  
+  # hist2(Pars$Kgrad, col=col, axes=FALSE, main="Kgrad", breaks=breaks)
+  # abline(v=Pars$Kgrad[its], col=1:nsamp, lwd=lwd)
+  # axis(side=1)  
   
   
   
@@ -1335,8 +1328,8 @@ plotGrowth <- function(OM, Pars=NULL, nsim=48, nyears=50, proyears=50, nsamp=3,
   
   
   # Growth curves
-  Len_age <- Pars$Len_age
-  Wt_age <- Pars$Wt_age
+  Len_age <- Pars$Length
+  Wt_age <- Pars$Weight
   cex.lab <- 1.25
   fstYr <- Len_age[its,,1]
   curYr <- Len_age[its,,nyears]
@@ -1436,11 +1429,8 @@ plotRec <- function(OM, Pars=NULL, nsim=48, nyears=50, proyears=50, nsamp=3, col
     hist2(0, col=col, axes=FALSE, main="Amplitude", breaks=breaks)
   }
   
-  
   # Recruitment
-  matplot(t(Pars$Perr_y[its,]), type="l", bty="l", main="Rec Devs by Year", lwd=lwd, lty=1, ylab="")
-  
-  
+  matplot(t(Pars$RecDev[its,]), type="l", bty="l", main="Rec Devs by Year", lwd=lwd, lty=1, ylab="")
 }
 
 
@@ -1487,21 +1477,21 @@ plotM2 <- function(OM, Pars=NULL, nsim=48, nyears=50, proyears=50, nsamp=3, col=
   abline(v=Pars$Msd[its], col=1:nsamp, lwd=lwd)
   axis(side=1) 
   
-  hist2(Pars$Mgrad, col=col, axes=FALSE, main="Mgrad", breaks=breaks)
-  abline(v=Pars$Mgrad[its], col=1:nsamp, lwd=lwd)
-  axis(side=1)
-  
+  # hist2(Pars$Mgrad, col=col, axes=FALSE, main="Mgrad", breaks=breaks)
+  # abline(v=Pars$Mgrad[its], col=1:nsamp, lwd=lwd)
+  # axis(side=1)
+  # 
   # M by year 
-  ylims <- range(Pars$M_ageArray[its,, ]) * c(0.95, 1.05)
+  ylims <- range(Pars$N.Mortality[its,, ]) * c(0.95, 1.05)
   ylims[1] <- min(0, ylims[1] )
-  matplot(t(Pars$Marray[its,]), type="l", lty=1, bty="l", main="average adult M by Year", lwd=lwd, ylab="M", ylim=ylims)
+  matplot(t(Pars$Marray[,its]), type="l", lty=1, bty="l", main="average adult M by Year", lwd=lwd, ylab="M", ylim=ylims)
   abline(v=nyears, col="gray", lty=2)
-  text(nyears, min(Pars$Marray[its,]), "Last historical year", pos=4, col="gray")
+  text(nyears, min(Pars$Marray[,its]), "Last historical year", pos=4, col="gray")
   
   # M at age 
-  M_ageArray <- Pars$M_ageArray
-  Len_age <- Pars$Len_age
-  Wt_at_age <- Pars$Wt_age
+  M_ageArray <- Pars$N.Mortality
+  Len_age <- Pars$Length
+  Wt_at_age <- Pars$Weight
   
   
   matplot(t(M_ageArray[its,,1]), type="l", lty=1, bty="l", lwd=lwd, ylim=ylims, ylab="M")
@@ -1704,7 +1694,7 @@ Data_xl <- function(fname, stkname, fpath = "", saveCSV = FALSE) {
   # Data
   index <- which(pmatch(shtname, paste0(stkname, "Data")) == 1)
   if (length(index) > 1)  stop("More than one match")
-  data <- readxl::read_excel(infile, sheet = index, col_names = FALSE)
+  data <- suppressMessages(readxl::read_excel(infile, sheet = index, col_names = FALSE))
   data <- as.data.frame(data)
   tmpfile <- paste0(fpath, stkname, "Data.csv")
   if (file.exists(tmpfile)) unlink(tmpfile)
