@@ -35,7 +35,6 @@ arma::mat popdynOneTScpp(double nareas, double maxage, Rcpp::NumericVector SSBcu
   // arma::mat tempMat2(nareas, nareas);	
   arma::mat Nstore(maxage, nareas); 
 
-  
   // Recruitment assuming regional R0 and stock wide steepness
   for (int A=0; A < nareas; A++) {
     if (SRrel == 1) {
@@ -52,8 +51,7 @@ arma::mat popdynOneTScpp(double nareas, double maxage, Rcpp::NumericVector SSBcu
       Nnext(age, A) = Ncurr(age-1, A) * exp(-Zcurr(age-1, A)); // Total mortality
     }
   }
-  // 
-  
+
   // Move stock
   for (int age=0; age<maxage; age++) {
   
@@ -92,7 +90,7 @@ arma::mat popdynOneTScpp(double nareas, double maxage, Rcpp::NumericVector SSBcu
 //' @param Vuln Numeric matrix (maxage, pyears) with vulnerability by age and year
 //' @param Retc Numeric matrix (maxage, pyears) with retention by age and year
 //' @param Prec Numeric vector (pyears) with recruitment error
-//' @param mov Numeric matrix (nareas by nareas) with the movement matrix
+//' @param movc Numeric array (nareas by nareas) with the movement matrix
 //' @param SRrelc Integer indicating the stock-recruitment relationship to use (1 for Beverton-Holt, 2 for Ricker)
 //' @param Effind Numeric vector (length pyears) with the fishing effort by year
 //' @param Spat_targc Integer. Spatial targetting
@@ -115,7 +113,7 @@ arma::mat popdynOneTScpp(double nareas, double maxage, Rcpp::NumericVector SSBcu
 List popdynCPP(double nareas, double maxage, arma::mat Ncurr, double pyears,
                arma::mat M_age, arma::vec Asize_c, arma::mat MatAge, arma::mat WtAge,
                arma::mat Vuln, arma::mat Retc, arma::vec Prec,
-               arma::cube movc, double SRrelc, arma::vec Effind,
+               List movc, double SRrelc, arma::vec Effind,
                double Spat_targc, double hc, NumericVector R0c, NumericVector SSBpRc,
                NumericVector aRc, NumericVector bRc, double Qc, double Fapic, double maxF, 
                arma::mat MPA, int control, double SSB0c) {
@@ -141,10 +139,13 @@ List popdynCPP(double nareas, double maxage, arma::mat Ncurr, double pyears,
   NumericVector SSB0a(nareas);
   double R0 = sum(R0c);
 
-  
   // Initial year
   Narray.subcube(0, 0, 0, maxage-1, 0, nareas-1) = Ncurr;
   
+
+  int yr = 0;
+  arma::cube movcy = movc(yr);
+  
   for (int A=0; A<nareas; A++) {
     Barray.subcube(0, 0, A, maxage-1, 0, A) = Ncurr.col(A) % WtAge.col(0);
     SSNarray.subcube(0, 0, A, maxage-1, 0, A) = Ncurr.col(A) % MatAge.col(0); 
@@ -154,7 +155,6 @@ List popdynCPP(double nareas, double maxage, arma::mat Ncurr, double pyears,
     tempVec(A) = accu(VBarray.slice(A));
   }
   
-
   // fishdist = (pow(tempVec, Spat_targc))/mean((pow(tempVec, Spat_targc)));
   fishdist = (pow(tempVec, Spat_targc))/sum((pow(tempVec, Spat_targc)));
 
@@ -172,8 +172,6 @@ List popdynCPP(double nareas, double maxage, arma::mat Ncurr, double pyears,
     }
   }
   
- 
-  
   // apply Fmax condition 
   arma::uvec tempvals = arma::find(FMarray > (1-exp(-maxF)));
   FMarray.elem(tempvals).fill(1-exp(-maxF));
@@ -181,23 +179,21 @@ List popdynCPP(double nareas, double maxage, arma::mat Ncurr, double pyears,
   FMretarray.elem(tempvals2).fill(1-exp(-maxF));
   
   Zarray.subcube(0,0, 0, maxage-1, 0, nareas-1) = Marray.subcube(0,0, 0, maxage-1, 0, nareas-1) + FMarray.subcube(0,0, 0, maxage-1, 0, nareas-1);
-  
 
   for (int yr=0; yr<(pyears-1); yr++) {
     // Rcpp::Rcout << "yr = " << yr << std::endl;
     arma::vec SB(nareas);
+    arma::cube movcy = movc(yr);
     
     for (int A=0; A<nareas; A++) SB(A) = accu(SBarray.subcube(0, yr, A, maxage-1, yr, A));
     if ((yr >0) & (control==3)) SB = SSB0a;
           
-          
     arma::mat Ncurr2 = Narray.subcube(0, yr, 0, maxage-1, yr, nareas-1);
     arma::mat Zcurr = Zarray.subcube(0, yr, 0, maxage-1, yr, nareas-1);
     
     arma::mat NextYrN = popdynOneTScpp(nareas, maxage, wrap(SB), wrap(Ncurr2), wrap(Zcurr), 
-                                       Prec(yr+1+maxage), hc, R0c2, SSBpRc, aRc2, bRc2, movc, SRrelc); 
+                                       Prec(yr+1+maxage), hc, R0c2, SSBpRc, aRc2, bRc2, movcy, SRrelc); 
   
-
     Narray.subcube(0, yr+1, 0, maxage-1, yr+1, nareas-1) = NextYrN;
     for (int A=0; A<nareas; A++) {
       Barray.subcube(0, yr+1, A, maxage-1, yr+1, A) = NextYrN.col(A) % WtAge.col(yr+1);
